"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const escapeComment_1 = __importDefault(require("./escapeComment"));
const escapeFieldName_1 = __importDefault(require("./escapeFieldName"));
const escapeIdentifier_1 = __importDefault(require("./escapeIdentifier"));
const ImportGenerator_1 = __importDefault(require("./ImportGenerator"));
const processComments = (comments, indentation) => {
    if (!comments || comments.length === 0) {
        return [];
    }
    const escapedComments = comments.map((comment) => (0, escapeComment_1.default)(comment));
    const i = " ".repeat(indentation);
    if (escapedComments.length === 1) {
        return [`${i}/** ${escapedComments[0]} */`];
    }
    else {
        const lines = [];
        lines.push(`${i}/**`);
        escapedComments.forEach((comment) => lines.push(`${i} * ${comment}`));
        lines.push(`${i} */`);
        return lines;
    }
};
const processDeclaration = (declaration, importGenerator) => {
    const declarationLines = [];
    switch (declaration.declarationType) {
        case "typeDeclaration": {
            const { exportAs, name, typeDefinition, comment } = declaration;
            declarationLines.push(...processComments(comment, 0));
            if (exportAs === "default") {
                if (typeDefinition.length === 1) {
                    declarationLines.push(`type ${(0, escapeIdentifier_1.default)(name)} = ${typeDefinition[0]};`);
                }
                else {
                    const [head, ...tail] = typeDefinition;
                    const tailLines = tail.map((l, i) => i === tail.length - 1 ? `  ${l};` : `  ${l}`);
                    declarationLines.push(`type ${(0, escapeIdentifier_1.default)(name)} = ${head}`, ...tailLines);
                }
                declarationLines.push("", `export default ${(0, escapeIdentifier_1.default)(name)};`);
            }
            else {
                if (typeDefinition.length === 1) {
                    declarationLines.push(`export type ${(0, escapeIdentifier_1.default)(name)} = ${typeDefinition[0]};`);
                }
                else {
                    const [head, ...tail] = typeDefinition;
                    const tailLines = tail.map((l, i) => i === tail.length - 1 ? `  ${l};` : `  ${l}`);
                    declarationLines.push(`export type ${(0, escapeIdentifier_1.default)(name)} = ${head}`, ...tailLines);
                }
            }
            break;
        }
        case "interface": {
            const { exportAs, name, base, properties, comment } = declaration;
            declarationLines.push(...(processComments(comment, 0) || []), `export${exportAs === "default" ? " default" : ""} interface ${(0, escapeIdentifier_1.default)(name)}${base ? ` extends ${base}` : ""} {`);
            properties.forEach((property, index) => {
                if (index > 0) {
                    declarationLines.push("");
                }
                declarationLines.push(...(processComments(property.comment, 2) || []), `  ${(0, escapeFieldName_1.default)(property.name)}${property.isOptional ? "?" : ""}: ${property.typeName}${"[]".repeat(property.dimensions)}${property.isNullable ? " | null" : ""};`);
                if (property.typeImports) {
                    property.typeImports.forEach((typeImport) => importGenerator.addImport(typeImport));
                }
            });
            declarationLines.push("}");
            break;
        }
        case "enum": {
            const { exportAs, name, values, comment } = declaration;
            declarationLines.push(...(processComments(comment, 0) || []), exportAs === "named"
                ? `export enum ${(0, escapeIdentifier_1.default)(name)} {`
                : `enum ${(0, escapeIdentifier_1.default)(name)} {`, ...values.map((value) => `  ${(0, escapeFieldName_1.default)(value)} = '${value}',`), "};");
            if (exportAs === "default") {
                declarationLines.push("", `export default ${(0, escapeIdentifier_1.default)(name)};`);
            }
            break;
        }
        case "constant": {
            const { exportAs, name, type, value, comment } = declaration;
            const values = Array.isArray(value) ? value : [value];
            const valuesWithSemicolon = values.map((v, i) => i === values.length - 1 ? `${v};` : v);
            const [valueHead, ...valueTail] = valuesWithSemicolon;
            declarationLines.push(...(processComments(comment, 0) || []), `export${exportAs === "default" ? " default" : ""} const ${(0, escapeIdentifier_1.default)(name)}${type ? `: ${type}` : ""} = ${valueHead}`, ...valueTail);
            break;
        }
        case "generic": {
            declarationLines.push(...processComments(declaration.comment, 0), ...declaration.lines);
            break;
        }
        default: {
            throw new Error(`Unknown declaration type: ${declaration.declarationType}`);
        }
    }
    if (declaration.typeImports) {
        declaration.typeImports.forEach((typeImport) => importGenerator.addImport(typeImport));
    }
    return declarationLines;
};
const render = (declarations, outputPath) => {
    const importGenerator = new ImportGenerator_1.default(outputPath);
    const lines = [];
    declarations.forEach((declaration, index) => {
        if (index > 0) {
            lines.push("");
        }
        const declarationLines = processDeclaration(declaration, importGenerator);
        lines.push(...declarationLines);
    });
    const importLines = importGenerator.generateLines();
    if (importLines.length > 0) {
        importLines.push("");
    }
    return [...importLines, ...lines];
};
exports.default = render;
